﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Irony.Interpreter.Ast;

namespace DeyjaScript
{
    class BinaryOperationNode : ExpressionNode
    {
        SystemFunction opFunc;

        public override void Init(Irony.Parsing.ParsingContext context, Irony.Parsing.ParseTreeNode treeNode)
        {
            base.Init(context, treeNode);
            AddChild("Parameter", treeNode.ChildNodes[0]);
            AddChild("Parameter", treeNode.ChildNodes[2]);
            this.AsString = treeNode.ChildNodes[1].FindTokenAndGetText();
        }

        internal override string getResultType(ExecutionContext context)
        {
            var func = context.lookupFunction(AsString, (ChildNodes[0] as ExpressionNode).getResultType(context),
                (ChildNodes[1] as ExpressionNode).getResultType(context));
            if (func == null) throw new InvalidOperationException("Operator not defined");
            return func.returnType;
        }

        public override void gatherInformation(ExecutionContext context)
        {
            opFunc = context.lookupFunction(AsString, (ChildNodes[0] as ExpressionNode).getResultType(context),
                (ChildNodes[1] as ExpressionNode).getResultType(context));
            if (opFunc == null) throw new CompileError("Operator not defined");
            (ChildNodes[0] as ExpressionNode).gatherInformation(context);
            (ChildNodes[1] as ExpressionNode).gatherInformation(context);
        }

        public override void emitByteCode(List<byte> bytecode, ExecutionContext context)
        {
            (ChildNodes[0] as ExpressionNode).emitByteCode(bytecode, context);
            (ChildNodes[1] as ExpressionNode).emitByteCode(bytecode, context);
            bytecode.Add((byte)Instruction.systemcall);
            bytecode.AddRange(BitConverter.GetBytes(opFunc.funcTableId));
        }
    }

    
}
