﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DeyjaScript
{
    public class ExecutionContext
    {
        internal Scope currentScope;

        internal Dictionary<string, SystemFunction> systemFunctions = new Dictionary<string, SystemFunction>();
        internal List<SystemFunction> systemFunctionTable = new List<SystemFunction>();

        internal String decorateFunctionName(String name, params String[] parameterTypes)
        {
            return name + String.Join("@", parameterTypes);
        }

        public void addSystemFunction(
            String name,
            String returnType,
            Func<object[], object> implementation,
            params String[] parameterTypes)
        {
            var systemFunction = new SystemFunction();
            systemFunction.name = name;
            systemFunction.returnType = returnType;
            systemFunction.parameterTypes = parameterTypes;
            systemFunction.systemFunc = implementation;
            systemFunctions.Add(decorateFunctionName(name, parameterTypes), systemFunction);

            systemFunction.funcTableId = systemFunctionTable.Count;
            systemFunctionTable.Add(systemFunction);
        }

        internal SystemFunction lookupFunction(String name, params String[] parameterTypes)
        {
            var key = decorateFunctionName(name, parameterTypes);
            if (systemFunctions.ContainsKey(key)) return systemFunctions[key];
            else return null;
        }
    }
}
