﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Irony.Parsing;
using Irony.Interpreter.Ast;

namespace DeyjaScript
{
    [Language("DeyjaScript", "0.1", "Scripting language for GDnet article series")]
    public class Grammar : Irony.Parsing.Grammar
    {
        public Grammar()
        {
            this.LanguageFlags |= Irony.Parsing.LanguageFlags.CreateAst;

            var numberLiteral = TerminalFactory.CreateCSharpNumber("number");
            numberLiteral.AstNodeType = typeof(IntLiteralNode);
            var identifier = TerminalFactory.CreateCSharpIdentifier("identifier");
            identifier.AstNodeType = typeof(IdentifierNode);

            var expression = new NonTerminal("Expression");
            var parenExpression = new NonTerminal("Paren Expression");
            var binaryOperation = new NonTerminal("Binary Operation", typeof(BinaryOperationNode));
            var @operator = ToTerm("+") | "-" | "*" | "/";
            var variableDeclaration = new NonTerminal("Variable Declaration", typeof(VariableDeclarationNode));
            var statement = new NonTerminal("Statement");
            var statementBlock = new NonTerminal("Statement Block", typeof(StatementBlockNode));
            var assignment = new NonTerminal("Assignment", typeof(AssignmentNode));

            expression.Rule = binaryOperation | numberLiteral | parenExpression | identifier;
            assignment.Rule = identifier + "=" + expression;
            binaryOperation.Rule = expression + @operator + expression;
            parenExpression.Rule = ToTerm("(") + expression + ")";
            variableDeclaration.Rule = identifier + identifier + ";";
            statement.Rule = (assignment + ";") | variableDeclaration;
            statementBlock.Rule = MakeStarRule(statementBlock, statement);

            this.Root = statementBlock;

            this.Delimiters = "{}[](),:;+-*/%&|^!~<>=";
            this.MarkPunctuation(";", ",", "(", ")", "{", "}", "[", "]", ":");
            this.MarkTransient(expression, parenExpression, statement);

            this.RegisterOperators(1, Associativity.Left, "+", "-");
            this.RegisterOperators(2, Associativity.Left, "*", "/");
            this.RegisterOperators(3, Associativity.Right, "=");
        }

    }
}