﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Irony.Interpreter.Ast;

namespace DeyjaScript
{
    class StatementBlockNode : ExpressionNode
    {
        internal Scope thisScope = new Scope();

        public override void Init(Irony.Parsing.ParsingContext context, Irony.Parsing.ParseTreeNode treeNode)
        {
            base.Init(context, treeNode);
            foreach (var f in treeNode.ChildNodes)
                AddChild("Statement", f);
        }

        public override void gatherInformation(ExecutionContext context)
        {
            var currentScope = context.currentScope;
            context.currentScope = thisScope;
            foreach (var statement in ChildNodes)
                (statement as ExpressionNode).gatherInformation(context);
            context.currentScope = currentScope;            
        }

        public override void emitByteCode(List<byte> bytecode, ExecutionContext context)
        {
            bytecode.Add((byte)Instruction.pushN);
            bytecode.Add((byte)thisScope.variables.Count);
            foreach (var statement in ChildNodes)
                (statement as ExpressionNode).emitByteCode(bytecode, context);
            bytecode.Add((byte)Instruction.popN);
            bytecode.Add((byte)thisScope.variables.Count);
        }
    }
}
