﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DeyjaScript
{
    public class VirtualMachine
    {
        internal Stack stack = new Stack();

        public object debugStackFetch(int index)
        {
            return stack.fetch(index);
        }



        public void execute(List<byte> bytecode, ExecutionContext context)
        {
            stack.clear();
            int instructionPointer = 0;

            while (instructionPointer < bytecode.Count)
            {
                switch ((Instruction)bytecode[instructionPointer])
                {
                    case Instruction.pushInt:
                        {
                            var _int = BitConverter.ToInt32(bytecode.GetRange(instructionPointer + 1, 4).ToArray(), 0);
                            stack.push(_int);
                            instructionPointer += 5;
                        }
                        break;
                    case Instruction.systemcall:
                        {
                            var funcID = BitConverter.ToInt32(bytecode.GetRange(instructionPointer + 1, 4).ToArray(), 0);
                            var func = context.systemFunctionTable[funcID];
                            var parameterArray = new object[func.parameterTypes.Length];
                            for (int i = 0; i < parameterArray.Length; ++i)
                                parameterArray[i] = stack.fetch(stack.topPointer - parameterArray.Length + i);
                            for (int i = 0; i < parameterArray.Length; ++i)
                                stack.pop();
                            stack.push(func.systemFunc(parameterArray));
                            instructionPointer += 5;
                        }
                        break;
                    case Instruction.pushN:
                        {
                            var count = bytecode[instructionPointer + 1];
                            stack.pushN(count);
                            instructionPointer += 2;
                        }
                        break;
                    case Instruction.popN:
                        {
                            var count = bytecode[instructionPointer + 1];
                            stack.popN(count);
                            instructionPointer += 2;
                        }
                        break;
                    case Instruction.pushVar:
                        {
                            var variableIndex = bytecode[instructionPointer + 1];
                            var variable = stack.fetch(stack.framePointer + variableIndex);
                            stack.push(variable);
                            instructionPointer += 2;
                        }
                        break;
                    case Instruction.popVar:
                        {
                            var variableIndex = bytecode[instructionPointer + 1];
                            var variable = stack.fetch(stack.topPointer - 1);
                            stack.store(stack.framePointer + variableIndex, variable);
                            instructionPointer += 2;
                        }
                        break;
                    default:
                        throw new NotImplementedException();
                }
            }
        }
    }
}
