﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DeyjaScriptTest
{
    class Program
    {
        static void Main(string[] args)
        {
            var grammar = new DeyjaScript.Grammar();
            var parser = new Irony.Parsing.Parser(grammar);
            var program = parser.Parse("int foo; foo = 6 * 6;");
            var rootNode = program.Root.AstNode as DeyjaScript.ExpressionNode;

            var bytecode = new List<byte>();
            var context = new DeyjaScript.ExecutionContext();

            context.addSystemFunction("*", "int", (parms) => { return (parms[0] as int?).Value * (parms[1] as int?).Value; }, "int", "int");
            context.addSystemFunction("/", "int", (parms) => { return (parms[0] as int?).Value / (parms[1] as int?).Value; }, "int", "int");
            context.addSystemFunction("-", "int", (parms) => { return (parms[0] as int?).Value - (parms[1] as int?).Value; }, "int", "int");
            context.addSystemFunction("+", "int", (parms) => { return (parms[0] as int?).Value + (parms[1] as int?).Value; }, "int", "int");

            try
            {
                rootNode.gatherInformation(context);
                rootNode.emitByteCode(bytecode, context);
                Console.WriteLine(String.Join(" ", bytecode));
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            var VirtualMachine = new DeyjaScript.VirtualMachine();
            VirtualMachine.execute(bytecode, context);
            Console.WriteLine("Result : " + VirtualMachine.debugStackFetch(0).ToString());

            Console.ReadKey();
        }
    }
}
